/*
 *  Dialer Jtapi Application
 *  Version 1.0     2/98
 *
 *  This sample application places a telephone call using the core Call.connect() method. 
 *
 *  To execute, type:
 *
 *  java Dialer inputfile
 *
 *  where inputfile is a required parameter that indicates a filename
 *  containing login information and phone directory.  The format of the data in
 *  this file should be as follows:
 *
 *  [Login]
 *  tlink=lucent#cstaserv#csta#myserver
 *  hostname=myserver.lucent.com
 *  login=user1
 *  password=user1
 *  device=3331
 *  [End]
 *
 *  [Directory]
 *  Vincent=3331
 *  Marsellus=3332
 *  Jules=3333
 *  [End]
 *
 */

import java.awt.*;
import java.util.*;
import java.net.*;
import javax.telephony.*;
import javax.telephony.events.*;
import javax.telephony.capabilities.*;
import com.lucent.jtapi.tsapi.*;

import symantec.itools.awt.BorderPanel;
import symantec.itools.awt.ImagePanel;
import symantec.itools.awt.ImageButton;

/**
 * Class: Dialer
 *
 * Main class for the Dialer application.  It sets-up the user interface allows the
 * user to make calls and clear calls from a dircetory.
 *
 * @version 	1.0, 98/2
 * @author 	    Kelly Herrero
 */
 public class Dialer extends Frame implements CallObserver, ProviderObserver {
	
	// GUI Components
	private java.awt.MenuItem               exitMenuItem;
	private java.awt.MenuItem               loginMenuItem;
	private java.awt.Panel                  panel1;
	private symantec.itools.awt.BorderPanel borderPanel1;
	private boolean                         fComponentsAdjusted = false;

	// Jtapi objects
    private JtapiPeer                       jtapiPeer;
	private Provider                        provider;
	private Terminal                        myTerminal;
	private Address                         myAddress;
	
	// Application data
	private static final int                MAXITEMS = 3;
	private static final int                BWGIF = 0;
	private static final int                COLORGIF = 1;
	private static final int                NUMBER = 2;
	private static final int                NAME = 3;
	private static MyTextArea               trace;
	private static Hashtable                data;
	private static Hashtable                phoneDirectory;
    private static String                   dataArray[][] = {
                                            { "guy1_bw.gif", "guy1.gif", "3331", "guy1" },
                                            { "guy2_bw.gif", "guy2.gif", "3332", "guy2" },
                                            { "guy3_bw.gif", "guy3.gif", "3333", "guy3" }
                                            };
    private static DirectoryButton          dirButton[] = new DirectoryButton[Dialer.MAXITEMS];                                 
	
	
	void Login_Action ( Event event ) 
	{
		if ( ( loginMenuItem.getLabel() ).equals ( "Login..." ) ) 
		{
		    ( new LoginDialog ( this, "Tserver Login", true, trace, data )).show ();
		} 
		else 
		{
		    logout();
		}
	}

	void Exit_Action ( Event event ) 
	{
		logout ();
        hide ();                 // hide the Frame
        dispose ();              // free the system resources
        System.exit (0);         // close the application
	}

	public Dialer () 
	{
	    java.awt.MenuBar    mainMenuBar;
	    java.awt.Menu       fileMenu;
		
		// Set up graphical user interface
		setLayout( null );
		addNotify();
		resize ( insets().left + insets().right + 350,insets().top + insets().bottom + 255 );
		setFont ( new Font( "Helvetica", Font.PLAIN, 11 ) );
		setBackground ( new Color(12632256) );
		panel1 = new java.awt.Panel();
		panel1.setLayout(null);
		panel1.reshape(insets().left + 0,insets().top + 0,350,238);
		add ( panel1 );
		trace = new MyTextArea();
		trace.reshape (0, 170, 343, 50);
		panel1.add ( trace );
		
		borderPanel1 = new symantec.itools.awt.BorderPanel();
		try 
		{
			borderPanel1.setBevelStyle(symantec.itools.awt.BorderPanel.BEVEL_LOWERED);
		}
		catch(java.beans.PropertyVetoException e) { }
		try 
		{
			borderPanel1.setLabel ( "Directory" );
		}
		catch ( java.beans.PropertyVetoException e ) { }
		try 
		{
			borderPanel1.setAlignStyle ( symantec.itools.awt.BorderPanel.ALIGN_LEFT );
		}
		catch ( java.beans.PropertyVetoException e ) { }
		borderPanel1.setLayout ( null );
		borderPanel1.setBounds ( 0, 15, 350, 168 );
		borderPanel1.setFont ( new Font("Helvetica", Font.PLAIN, 15) );
		borderPanel1.setBackground ( new Color(12632256) );
		panel1.add ( borderPanel1 );		
		setTitle ( "Jtapi Directory Dialer" );

		// Set up Menu
		mainMenuBar = new java.awt.MenuBar ();
		fileMenu = new java.awt.Menu ( "File" );
		loginMenuItem = new java.awt.MenuItem ( "Login..." );
		fileMenu.add ( loginMenuItem );
		fileMenu.addSeparator ();
		exitMenuItem = new java.awt.MenuItem( "Exit" );
		fileMenu.add ( exitMenuItem );
		mainMenuBar.add ( fileMenu );
		setMenuBar ( mainMenuBar );
		
		// Initialize JTAPI
		if ( !initJtapi() ) 
		{
		    trace.appendText( "Error: JtapiPeer could not be created.  Verify your Jtapi client install.\n\n");
	        loginMenuItem.disable();
	    }
		
		// Register listeners
		SymWindow aSymWindow = new SymWindow();
		this.addWindowListener(aSymWindow);
		
		// Draw diretory buttons with pictures
		drawDirectory();
	}
	
	/*
    * initJtapi()
    * This method uses the static class JtapiPeerFactory to obtain the
    * JtapiPeer.
    */
    public boolean initJtapi () 
    {

        // Create JtapiPeer object
	    // Get the default JTAPI implementation by sending null in getJtapiPeer
		// Lucent's implementation of the JtapiPeer is com.lucent.jtapi.tsapi.TsapiPeer
        try {
		    jtapiPeer = JtapiPeerFactory.getJtapiPeer ( null );
        }
	    catch ( JtapiPeerUnavailableException e ) {
	        trace.appendText( "JtapiPeerFactory.getJtapiPeer: caught JtapiPeerUnavailableException\n" );
            trace.appendText( "Message: " + e.getMessage() + "\n\n" );
	        e.printStackTrace();
	        return false;
	    }
	    return true;
    }
    
   /*
    * getServices()
    * This method uses the getServices() method in the JtapiPeer interface to get
    * a list of the advertised services (i.e. tlinks).
    */
    public String[] getServices () 
    {
	    String[]   services = null;

		// get available tserver links
        try {
			services = jtapiPeer.getServices();
		}
		catch ( PlatformException e ) {
	        trace.appendText( "JtapiPeer.getServices(): caught PlatformException\n" );
            trace.appendText( "Message: " + e.getMessage() + "\n\n" );
	        e.printStackTrace();
	        return null;
	    }
	    return services;
	}
	
   /**
    * drawDirectory()
    * This method is called when an argument has been specified when the program
    * was called.  This argument is a filename with default values of login information.
    * It uses the Preferences class to invoke the getData method which returns a
    * Hashtable containing parameter-value pairs
    */
    public void drawDirectory () 
    {
		SymMouse aSymMouse = new SymMouse();
		
		dirButton[0].setBounds (   2, 7, 106, 110 );
		dirButton[1].setBounds ( 110, 7, 106, 110 );
		dirButton[2].setBounds ( 218, 7, 106, 110 );
		
		for ( int i = 0; i < Dialer.MAXITEMS; i++ )
		{
		    try {
			    dirButton[i].setImageURL( dirButton[i].getbwURL() );
		    }
		    catch ( java.beans.PropertyVetoException e ) { }
		    
		    borderPanel1.add ( dirButton[i] );
		    dirButton[i].disable ();
		    
		    // register listeners
		    dirButton[i].addMouseListener(aSymMouse);
		}		
    }
	

    public synchronized void show () 
    {
    	move ( 50, 50 );
    	super.show();
    }

	public boolean handleEvent ( Event event ) 
	{
    	if ( event.id == Event.WINDOW_DESTROY ) 
    	{
            Exit_Action ( event );
            return true;
    	}
		if ( event.target == this && event.id == Event.GOT_FOCUS ) 
		{
			toFront();
			return true;
		}
		return super.handleEvent ( event );
	}

	public boolean action ( Event event, Object arg ) 
	{
		if (event.target instanceof MenuItem) 
		{
		    if ( event.target == loginMenuItem )
		    {
		        Login_Action ( event );
			    return true;
			}
			else if ( event.target == exitMenuItem )
		    {
		        Exit_Action ( event );
			    return true;
			}
		}
		return super.action ( event, arg );
	}

   /**
    * login()
    * This method uses the core JTAPI package to:
    *
    * 1) create Provider object
    * 2) register observer for Provider
    * 3) obtain references to Terminal and Address for device specified
    * 4) register observer for Address
    * 5) determine capabilities
    */
    public synchronized void login ( Hashtable args ) 
    {
        String                  serviceID;
        String                  login;
        String                  password;
        String                  providerString;
        String                  callingExt;
        ProviderCapabilities    pCapabilities = null;
        CallCapabilities        cCapabilities = null;
        TerminalCapabilities    tCapabilities = null;
        
        serviceID   = (String) args.get( "serviceID" );
		login       = (String) args.get( "login" );
		password    = (String) args.get( "password" );
        callingExt  = (String) args.get( "callingExt" );

		provider = null;
                
        // Lucent's implementation requires these fields in order to obtain the Provider:
        // tlink name, login and password.
        // serviceID is one of the strings returned by getServices()
        providerString = serviceID + ";login=" + login + ";passwd=" + password;

		// create Provider object
        try 
        {
			provider = jtapiPeer.getProvider( providerString );
		}
		catch (PlatformException e) {
	        trace.appendText( "JtapiPeer.createProvider(): caught PlatformException\n" );
            trace.appendText( "Message: " + e.getMessage() + "\n\n" );
    	    trace.appendText( "Error: Provider could not be created\n\n" );
	        e.printStackTrace();
	        return;
	    }
        
	    trace.appendText( "Provider created successfully\n\n" );
        
        // query the capabilities of the Provider to find out
		// if we are allowed to observe it
		pCapabilities = provider.getCapabilities();
		
	    if ( pCapabilities.isObservable() )
		{
    		// Add an observer to the Provider to monitor state changes --
    		// note that this class implements the ProviderObserver
    		try 
    		{
    		    provider.addObserver( this );
    		}
    		catch ( ResourceUnavailableException e ) 
    	    {
    	        trace.appendText( "Provider.addObserver(): caught ResourceUnavailableException\n" );
                trace.appendText( "Message: " + e.getMessage() + "\n\n" );
    	        e.printStackTrace();
            }
        }
        else
        {
            trace.appendText( "Warning: Unable to observe Provider " + provider.getName() + "\n\n" );
        }
        
        // Initially, provider may be OUT_OF_SERVICE while it initializes --
        // wait for it to be IN_SERVICE
        trace.appendText( "Waiting for the provider to be in service" );
	    
	    try 
	    {
	        // wait to be notified when provider is in service --
	        // corresponding notify is in the providerChangedEvent() method
	        wait ();                                          
	        trace.appendText( "." );
	    }
	    catch ( InterruptedException e ) 
	    {
	        trace.appendText( "wait(): caught InterruptedException\n" );
        }	        
		
		trace.appendText( "\nProvider is in service\n\n" );
		
        // Query static capabilities of the Call to find out if we are
		// able to place calls -- necessary for a dialer type application
		cCapabilities = provider.getCallCapabilities( );
		
		if ( ! cCapabilities.canConnect () )
		{
            trace.appendText( "Error: Provider is unable to place calls.\n\n" );
            return;
        }
        
        // obtain a reference to the Address associated with the device specified
        try 
        {
            myAddress = provider.getAddress( callingExt );
		    trace.appendText( "Address " + callingExt + " created successfully\n\n" );
        }
	    catch ( InvalidArgumentException e ) 
	    {
	        trace.appendText( "Provider.getAddress(): caught InvalidArgumentException\n" );
            trace.appendText( "Message: " + e.getMessage() + "\n\n" );
	        e.printStackTrace();
	        return;
        }
	    catch ( PlatformException e ) 
	    {
	        trace.appendText( "Provider.getAddress(): caught PlatformException\n" );
            trace.appendText( "Message: " + e.getMessage() + "\n\n" );
	        e.printStackTrace();
	        return;
        }
                
        // obtain a reference to the Terminal associated with the device specified
        try 
        {
    	    myTerminal = provider.getTerminal( callingExt );
		    trace.appendText( "Terminal " + callingExt + " created successfully\n\n" );
        }
	    catch ( InvalidArgumentException e ) 
	    {
	        trace.appendText( "Provider.getTerminal(): caught InvalidArgumentException\n" );
            trace.appendText( "Message: " + e.getMessage() + "\n\n" );
	        e.printStackTrace();
	        return;
        }
	    catch (PlatformException e) 
	    {
	        trace.appendText( "Provider.getTerminal(): caught PlatformException\n" );
            trace.appendText( "Message: " + e.getMessage() + "\n\n" );
	        e.printStackTrace();
	        return;
        }

        trace.appendText( "Logged in successfully...\n\n" );

        // query the capabilities of the Terminal to find out
		// if we will be able to observe it
		tCapabilities = myTerminal.getCapabilities( );
		
    	if ( tCapabilities.isObservable() )
		{
        	// Add CallObserver to Terminal so that the implementation automatically add
            // a CallObserver for every call on the Terminal --
            // note that this class implements the CallControlCallObserver
    		try 
    		{
                myTerminal.addCallObserver( this );
            }
            catch ( ResourceUnavailableException e ) 
            {
    	        trace.appendText( "Terminal.addCallObserver(): caught ResourceUnavailableException\n" );
                trace.appendText( "Message: " + e.getMessage() + "\n\n" );
    	        e.printStackTrace();
    	        return;
            }
    	    catch ( PlatformException e ) 
    	    {
    	        trace.appendText( "Terminal.addCallObserver(): caught PlatformException\n" );
                trace.appendText( "Message: " + e.getMessage() + "\n\n" );
    	        e.printStackTrace();
    	        return;
            }
        }
        else 
        {
            trace.appendText( "Error: Unable to observe terminal " + myTerminal.getName() + "\n\n" );
            return;
        }
        
        // Only for the purpose of this demo, clear all calls at that device --
        // normally, current calls at the device should NOT be disconnected.
        disconnect();
        
        // Now that we havelogged in successfuly, make appropriate changes
        // to the user interface (e.g. enable buttons, reset menus, etc)
        for (int i = 0; i < Dialer.MAXITEMS; i++ )
		{
		    dirButton[i].enable();
		}
		loginMenuItem.setLabel( "Logout..." );
    }

    
   /**
    * makeCall() - makes a call using the Call.connect() method in the core
    */
    public boolean makeCall ( String calledNumber ) {
        Call                    call = null;
        Connection[]            connections = null;
        boolean                 rc = false;
        CallCapabilities        cCapabilities = null;
        
        try {
            // create Call object
            call = provider.createCall ();
        }
	    catch (InvalidStateException e) {
	        trace.appendText ( "Provider.createCall(): caught InvalidStateException\n" );
            trace.appendText ( "Message: " + e.getMessage() + "\n\n" );
	        e.printStackTrace();
	        return rc;
        }
	    catch (MethodNotSupportedException e) {
	        trace.appendText ( "Provider.createCall(): caught MethodNotSupportedException\n" );
            trace.appendText ( "Message: " + e.getMessage() + "\n\n" );
	        e.printStackTrace();
	        return rc;
        }
	    catch (ResourceUnavailableException e) {
	        trace.appendText ( "Provider.createCall(): caught ResourceUnavailableException\n" );
            trace.appendText ( "Message: " + e.getMessage() + "\n\n" );
	        e.printStackTrace();
	        return rc;
        }
	    catch (PlatformException e) {
	        trace.appendText ( "Provider.createCall(): caught PlatformException\n" );
            trace.appendText ( "Message: " + e.getMessage() + "\n\n" );
	        e.printStackTrace();
	        return rc;
        }
	    catch ( PrivilegeViolationException e ) {
	        trace.appendText ( "Provider.createCall(): caught PrivilegeViolationException\n" );
            trace.appendText ( "Message: " + e.getMessage() + "\n\n" );
	        e.printStackTrace();
	        return rc;
        }

        try {
    	    // makecall by using the connect method in the LucentCall interface
            connections = call.connect( myTerminal, myAddress, calledNumber );
            rc = true;
    	}
        catch ( InvalidStateException e ) {
            trace.appendText ( "Call.connect: caught InvalidStateException\n" );
            trace.appendText ( "Message: " + e.getMessage() + "\n\n" );
	        e.printStackTrace();
        }
        catch ( MethodNotSupportedException e ) {
            trace.appendText ( "Call.connect: caught MethodNotSupportedException\n" );
            trace.appendText ( "Message: " + e.getMessage() + "\n\n" );
	        e.printStackTrace();
        }
        catch ( InvalidArgumentException e ) {
            trace.appendText ( "Call.connect: caught InvalidArgumentException\n" );
            trace.appendText ( "Message: " + e.getMessage() + "\n\n" );
	        e.printStackTrace();
        }
        catch ( ResourceUnavailableException e ) {
            trace.appendText ( "Call.connect: caught ResourceUnavailableException\n" );
            trace.appendText ( "Message: " + e.getMessage() + "\n\n" );
	        e.printStackTrace();
        }
        catch ( PlatformException e ) {
            trace.appendText ( "Call.connect: caught PlatformException\n" );
            trace.appendText ( "Message: " + e.getMessage() + "\n\n" );
	        e.printStackTrace();
        }
        catch ( PrivilegeViolationException e ) {
            trace.appendText ( "Call.connect: caught PrivilegeViolationException\n" );
            trace.appendText ( "Message: " + e.getMessage() + "\n\n" );
	        e.printStackTrace();
        }
        catch ( InvalidPartyException e ) {
            trace.appendText ( "Call.connect: caught InvalidPartyException\n" );
            trace.appendText ( "Message: " + e.getMessage() + "\n\n" );
	        e.printStackTrace();
        }
        return rc;
    }


   /**
    * disconnect() - Disconnects active call (in the TALKING state)
    * This method uses the getTerminalConnections method of the Terminal interface.
    * If there is only one terminalConnection, terminate that one.  Otherwise,
    * get a handle on the connection that is in the TALKING state.  When a connection
    * is found, then calls the disconnect(connection) method to end the call.
    */
    public boolean disconnect() 
    {
		TerminalConnection[]     terminalConnections = null;
		boolean                  foundConnection = false;
		Connection               connection;

        try {
            terminalConnections = myTerminal.getTerminalConnections();
        }
        catch ( PlatformException e ) {
            trace.appendText ( "Terminal.getTerminalConnections: caught PlatformException\n" );
            trace.appendText ( "Message: " + e.getMessage() + "\n\n" );
	        e.printStackTrace();
        }

        if ( terminalConnections != null ) 
        {
                for ( int i=0; i < terminalConnections.length; i++ ) 
                {
                        connection = terminalConnections[i].getConnection();
                        disconnect ( connection );
                        foundConnection = true;
                }
        }
        return foundConnection;
    }

   /**
    * disconnect(Connection) 
    * Disconnects the connection passed in argument using the disconnect()
    * method of the Connection interface to end the call.
    */
    public void disconnect( Connection connection ) {


        if ( connection != null ) {

            try {
                connection.disconnect();
            }
            catch ( PrivilegeViolationException e ) {
                trace.appendText ( "Connection.disconnect: caught PrivilegeViolationException\n" );
                trace.appendText ( "Message: " + e.getMessage() + "\n\n" );
    	        e.printStackTrace();
            }
            catch ( InvalidStateException e ) {
                trace.appendText ( "Connection.disconnect: caught InvalidStateException\n" );
                trace.appendText ( "Message: " + e.getMessage() + "\n\n" );
    	        e.printStackTrace();
            }
            catch ( MethodNotSupportedException e ) {
                trace.appendText ( "Connection.disconnect: caught MethodNotSupportedException\n" );
                trace.appendText ( "Message: " + e.getMessage() + "\n\n" );
    	        e.printStackTrace();
            }
            catch ( ResourceUnavailableException e ) {
                trace.appendText ( "Connection.disconnect: caught ResourceUnavailableException\n" );
                trace.appendText ( "Message: " + e.getMessage() + "\n\n" );
    	        e.printStackTrace();
            }
            catch ( PlatformException e ) {
                trace.appendText ( "Connection.disconnect: caught PlatformException\n" );
                trace.appendText ( "Message: " + e.getMessage() + "\n\n" );
    	        e.printStackTrace();
            }
        }
    }

   /**
    * logout() 
    * This method first verifies that the Provider is in service, it then removes the
    * callObserver that we have on the terminal, and then uses the shutdown method
    * on the Provider to terminate the session.
    */
    public void logout() 
    {
	    if ( provider.getState() == Provider.IN_SERVICE ) 
	    {
            // remove callObserver on terminal (nice but not necessary)
            if ( myTerminal != null ) 
            {
                try 
                {
                    CallObserver observers[] = myTerminal.getCallObservers();
                    if ( observers != null ) 
                    {
                        for ( int i=0; i < observers.length; i++ ) 
                        {
                            myTerminal.removeCallObserver ( observers[i] );
                        }
                    }
                }
                catch ( PlatformException e ) 
                {
                    trace.appendText ( "Terminal.removeCallObserver(): caught PlatformException\n" );
                    trace.appendText ( "Message: " + e.getMessage() + "\n\n" );
        	        e.printStackTrace();
                }
            }

            // shutdown Provider
            trace.appendText ( "Shutting down provider...\n\n" );
    		try 
    		{
    		    provider.shutdown();
    		}
            catch ( PlatformException e ) 
            {
                trace.appendText ( "ProviderFrame.shutdown(): caught PlatformException\n" );
                trace.appendText ( "Message: " + e.getMessage() + "\n\n" );
    	        e.printStackTrace();
            }
            finally {
                provider = null;
            }
        }

        // reset GUI and menus
        handleProviderShutdown ( null );
	}
	
   /**
    * handleConnDisconnected()
    * When a connection has gone away, disable the "Send DTMF" and the "Disconnect" menu
    * items; enable the "Make Call with UUI" menu item
    */
    public void handleConnDisconnected ( CallEv event ) 
    {
        Address address = ((ConnEv) event).getConnection().getAddress();
        
        if ( myAddress.equals ( address ) ) 
        {        
            Date d = new Date();

            // log connection going away
            trace.appendText ( d.toLocaleString() + " Connection Cleared.\n\n" );
                		
            for ( int i = 0; i < Dialer.MAXITEMS; i++ )
    		{
    	        try {
    				dirButton[i].setImageURL ( dirButton[i].getbwURL() );
    			}
    			catch ( java.beans.PropertyVetoException e ) { }
    		}
        }
    }

   /**
    * handleConnAlerting()
    * When a call has been placed from our observed terminal, record:
    *   - date and time call was placed
    *   - the called device information
    * Note that this method will be called for a specific event:
    *   - ConnAlertingEv - to track when connection is alerting at terminating
    *                             end of the call for internal calls
    */
    public void handleConnAlerting ( CallEv event ) 
    {
        Address         address;
        String          name = null;

        address = ((ConnEv) event).getConnection().getAddress();
        
        // verify this connection refers to the other party in the call
        if ( ! myAddress.equals ( address ) ) 
        {
            Date d = new Date();

            // get called number information
            if ( address instanceof LucentAddress ) 
            {
                name = ( (LucentAddress) address).getDirectoryName();
            }
            
            if ( name == null ) 
            {
                name = address.getName();
            }
            
            // log call
            trace.appendText ( d.toLocaleString() + " Call made to: " + name + " (" + address.getName() + ")\n\n" );
    		
    		for (int i = 0; i < Dialer.MAXITEMS; i++ )
    		{
    		    if ( (dirButton[i].getNumber()).equals ( address.getName() ) )
    		    {
    		        try {
        				dirButton[i].setImageURL( dirButton[i].getColorURL() );
        			}
        			catch(java.beans.PropertyVetoException e) { }
        			break;
            	}
    		}            		
    	}
    }

   /*
    * handleProviderShutdown()
    * This method does the necessary clean-up for our application, when the session has been
    * terminated.
    */
    public void handleProviderShutdown ( ProvEv event ) 
    {
        provider = null;
        
        loginMenuItem.setLabel("Login...");
        for (int i = 0; i < Dialer.MAXITEMS; i++ )
		{
		    try 
		    {
				dirButton[i].setImageURL( dirButton[i].getbwURL() );
			}
			catch(java.beans.PropertyVetoException e) { }
			
			dirButton[i].disable();
		}
    }
	
   /*
	* callChangedEvent()
	* This method is required to implement the CallControlCallObserver --
	* the implementation will call this routine to notify call state changes
	* associated with our Terminal.
	*/
	public void callChangedEvent ( CallEv[] eventList )
    {
        for ( int i = 0; i < eventList.length; i++ )
        {
            switch(eventList[i].getID())
            {
                case ConnAlertingEv.ID:      //handle connection alerting
    	            handleConnAlerting ( eventList[i] );
    	            break;
    	        case ConnDisconnectedEv.ID:      //handle call dropped
    	            handleConnDisconnected ( eventList[i] );
    	            break;
    	        default:
                	break;
		    }
		}
   	}
   	
   /*
	* providerChangedEvent()
	* This method is required to implement the ProviderObserver --
	* the implementation will call this routine to notify state changes
	* in the Provider.
	*/
	public synchronized void providerChangedEvent ( ProvEv[] eventList )
    {
	    for ( int i=0; i<eventList.length; i++ )
        {
	        switch(eventList[i].getID())
            {
                case ProvInServiceEv.ID:        //handle provider in service
    	            notify ();                  // login() is waiting on this event
            		break;
            	case ProvShutdownEv.ID:         //handle provider shutdown
    	            handleProviderShutdown ( eventList[i] );
            		break;
            	default:
            	    break;
		    }
		}
    }
	
    public void paint( Graphics d ) 
    {
    	Dimension size = size ();
    	panel1.resize ( size.width, size.height - 10 );
    	borderPanel1.reshape ( 0, 20, size.width - 7, 160 );
    	trace.resize ( size.width - 13, size.height - 200 );
    	return;
    }

   /**
    * readPreferences()
    * This method is called when an argument has been specified when the program
    * was called.  This argument is a filename with default values of login information.
    * It uses the Preferences class to invoke the getData method which returns a
    * Hashtable containing parameter-value pairs
    */
    public static void readPreferences ( String  filename ) 
    {
        Preferences prefs = new Preferences( trace );
	    data = prefs.getData ( filename, "[Login]" );
	    phoneDirectory = prefs.getData ( filename, "[Directory]" );
	    prefs.trimObjects ( phoneDirectory, " -()." );
	    
	    Enumeration names;
	    
	    names = phoneDirectory.keys();
	    if (names != null) {
	        int i = 0;
	        while ( names.hasMoreElements() && i < Dialer.MAXITEMS ) 
	        {
    	        String name   = (String) names.nextElement();
    	        String number = (String) phoneDirectory.get ( name );
    	        dirButton[i]  = new DirectoryButton( dataArray[i][Dialer.COLORGIF], 
    	                                             dataArray[i][Dialer.BWGIF],
    	                                             name,
    	                                             number );
    	        i++;
    	    }
    	}
    }


   /**
	* main()
	* Main method for the Java application.  Creates the Dialer application.
	*/
	static public void main ( String args[] ) 
	{
	    if ( args.length >= 1 ) 
	    {
	        readPreferences ( args[0] );
	    }
	    ( new Dialer() ).show ();
	}
	
	class SymWindow extends java.awt.event.WindowAdapter
	{
		public void windowClosing ( java.awt.event.WindowEvent event )
		{
			Object object = event.getSource();
			if (object == Dialer.this)
				Dialer_WindowClosing ( event );
		}
	}

	void Dialer_WindowClosing ( java.awt.event.WindowEvent event )
	{
		hide();		        // hide the Frame
		dispose();	        // free the system resources
		System.exit(0);     // close the application
	}
	
	public void addNotify ()
	{
	    // Record the size of the window prior to calling parents addNotify.
	    Dimension d = getSize();
	    
		super.addNotify ();

		if ( fComponentsAdjusted )
			return;

		// Adjust components according to the insets
		setSize( insets().left + insets().right + d.width, insets().top + insets().bottom + d.height );
		Component components[] = getComponents();
		for ( int i = 0; i < components.length; i++ )
		{
			Point p = components[i].getLocation ();
			p.translate ( insets().left, insets().top );
			components[i].setLocation ( p );
		}
		fComponentsAdjusted = true;
	}
	
	
	class SymMouse extends java.awt.event.MouseAdapter
	{
		public void mouseClicked ( java.awt.event.MouseEvent event )
		{
			b_mouseClicked ( event );
		}
		
		public void mouseExited ( java.awt.event.MouseEvent event )
		{
			b_mouseExited ( event );
		}

		public void mouseEntered ( java.awt.event.MouseEvent event )
		{
			b_mouseEntered ( event );
		}
	}

   /**
    * b_mouseClicked()
    * When a directory button has been selected, first verify if there are any active
    * calls by calling diconnect().  If there are no active calls, place call to the 
    * number that belongs to that directory entry, otherwise, just clear active calls.
    */
    void b_mouseClicked ( java.awt.event.MouseEvent event )
	{
		Object              object = event.getSource();
		DirectoryButton     imgButton;
		
		if ( object instanceof DirectoryButton ) 
		{
		    imgButton = (DirectoryButton) object;

			// Call disconnect to check and clear any active calls.  If there are no
			// active calls, then make call to entry selected
			if ( !disconnect () )
			{
			     makeCall ( imgButton.getNumber() );
			}
        }
	}
	
   /**
    * b_mouseEntered()
    * When the mouse enters the button, change border to color red
    */
    void b_mouseEntered ( java.awt.event.MouseEvent event )
	{
		Object              object = event.getSource();
		DirectoryButton     imgButton;
			 
		if ( object instanceof DirectoryButton ) {
		    imgButton = (DirectoryButton) object;

			try {
				imgButton.setBorderColor( Color.red );
			}
			catch ( java.beans.PropertyVetoException e ) { }
		}
	}
		
   /**
    * b_mouse_Exited()
    * When the mouse exits the button, change border to color black
    */
    void b_mouseExited ( java.awt.event.MouseEvent event )
	{
		Object              object = event.getSource();
		DirectoryButton     imgButton;
			 
		if ( object instanceof DirectoryButton ) {
		    imgButton = (DirectoryButton) object;

			try 
			{
				imgButton.setBorderColor( Color.black );
			}
			catch ( java.beans.PropertyVetoException e ) { }
		}
	}
}
